/**********************************************************************

S88.cpp
COPYRIGHT (c) 2013-2016 Xavier Bouillard

**********************************************************************/
/**********************************************************************

DCC++ BASE STATION supports S88 retrosignalisation.
This sensor data collection will be sent back as a binary string
this feedback will occure as soon as a change is detected. 
It will be necessary to initialize the S88 data collection indicating how many cards are connected (a card is a group of 8 detectors) using the following variation of the "Y" command:

  <Y Nb_S88_Modules DataFormat>:	sets Nb_S88_Modules read
				returns: <y status>
                
**********************************************************************/

#include "s88.h"
#include "SerialCommand.h"
#include "DCCpp_Uno.h"
#include "EEStore.h"
#include <EEPROM.h>
#include "Comm.h"

int NAdd;       // S88 modules quantity Addresse
int N;          // S88 modules quantity
int Nr;         // S88 Bits quantity = N*8
int DataFormat; // Output DataFormat 0=binary 1=hexa
String Old_Occ; // S88 detector previous status
String Occ;     // S88 detector building status

///////////////////////////////////////////////////////////////////////////////
  
long int S88::sampleTime=0;
long int S88::S88_Cpt=0;

boolean S88::checkTime(){
  if(micros()-sampleTime<S88_SAMPLE_TIME)           // no need to check S88 yet
    return(false);
  sampleTime=micros();                              // note millis() uses TIMER-0.
//  if(millis()-sampleTime<S88_SAMPLE_TIME)           // no need to check S88 yet
//    return(false);
//  sampleTime=millis();                              // note millis() uses TIMER-0.
  return(true);  
} // S88::checkTime
  

///////////////////////////////////////////////////////////////////////////////
void S88::check(){    
  S88 *tt;

  switch (S88_Cpt) {
    case 0:
      digitalWrite(S88_PS_PIN, LOW);
      digitalWrite(S88_Clock_PIN, LOW);
      digitalWrite(S88_Reset_PIN, LOW);
      Nr=N*8-1;
      Occ="";
      break;
    case 1:
      digitalWrite(S88_PS_PIN, HIGH);
      break;
    case 2:
      digitalWrite(S88_Clock_PIN, HIGH);
      break;
    case 3:
      Occ+=String(digitalRead(S88_Data_PIN));
      digitalWrite(S88_Clock_PIN, LOW);
      break;
    case 4:
      digitalWrite(S88_Reset_PIN, HIGH);
      break;
    case 5:
      digitalWrite(S88_Reset_PIN, LOW);
      break;
    case 6:
      digitalWrite(S88_PS_PIN, LOW);
      break;
    case 7:
      digitalWrite(S88_Clock_PIN, HIGH);
      break;
    case 8:
      Occ+=String(digitalRead(S88_Data_PIN));
      break;
    case 9:
      digitalWrite(S88_Clock_PIN, LOW);     
      if (Nr>1) {
        S88_Cpt=6;
        Nr--;
      } else {  // S88 string is ready, need to format it.
        if (Old_Occ!=Occ && Nr>0) { 
          Old_Occ=Occ;
          String S88_fb="<y ";
          if (DataFormat==0) INTERFACE.println();
          //INTERFACE.print("<y ");
          if (DataFormat==0) S88_fb += Occ; //INTERFACE.print(Occ);
          else {
            for (int i=0; i<Occ.length(); i=i+4) {
              String tmp=Occ.substring(i,i+4);
              int tmpint=0;
              for (int ii=0; ii < 4; ii++) {
                tmpint = tmpint | (tmp[ii] - '0') << ii;
              }
              if (tmpint<10)  S88_fb += tmpint; //INTERFACE.print(tmpint);
              else S88_fb += (char)(tmpint-10); //INTERFACE.print((char)(tmpint-10));
            }
          }
          S88_fb += ">";
          if (S88_fb.substring(S88_fb.length()-2,S88_fb.length()-1)!="") {     // Attention this is not a space character !
            INTERFACE.print(S88_fb);          
          } else {
            INTERFACE.print("<x S88 disconnected>");
          }
        }
        S88_Cpt=-1;
      }
      break;
  }
  S88_Cpt++;
} // Sensor::check

///////////////////////////////////////////////////////////////////////////////
// <Y Nb_S88_Modules>
void S88::parse(char *c){
  int n,f,m,df;
  S88 *t;
  
  switch(sscanf(c,"%d %d %d",&n,&f,&m)){
    
    case 1:                     // argument is string with Nb_S88_Modules (defauilt dataFormat is Hexa)
      N=n;                      // S88 modules quantity
      DataFormat=0;             // Output DataFormat 0=binary 1=hexa
      store();
      INTERFACE.print("<o " + String(N) + "*8 " + String(DataFormat) + ">");     // confirm init done
    break;

    case 2:                     // argument is string with Nb_S88_Modules and dataFormat
      if (f==0 || f==1) {
        N=n;                      // S88 modules quantity
        DataFormat=f;             // Output DataFormat 0=binary 1=hexa
        store();
        INTERFACE.print("<o " + String(N) + "*8 " + String(DataFormat) + ">");     // confirm init done
      } else {
        INTERFACE.print("<x Bad Argument value>"); // Bad Argument Value
      }
    break;

    case 3:                     // argument count incorrect (0 or 2)
      INTERFACE.print("<x Bad Argument count>");
    break;

    case -1:                    // no arguments: immediate last feedback
      // force occupation scan on next loop and feedback
      Old_Occ="";
      //check();
    break;
  }
}

///////////////////////////////////////////////////////////////////////////////

void S88::store(){
  EEStore::reset();
  S88 *tt;
//  INTERFACE.print("Store S88 ");
//  INTERFACE.print("<");
//  INTERFACE.print("N ");
//  INTERFACE.print(N);
//  INTERFACE.print(">");
//  INTERFACE.println();
  EEStore::eeStore->data.nS88=N;
  EEPROM.put(EEStore::pointer(),N); // Store Nr of S88 Modules
  EEStore::advance(1);
}

///////////////////////////////////////////////////////////////////////////////

void S88::load(){
  struct S88Data data;
  EEPROM.get(EEStore::pointer(), N);
  EEStore::advance(1);
//  INTERFACE.print("<Load S88 "); 
//  INTERFACE.print("N ");
//  INTERFACE.print(N);
//  INTERFACE.print(">");
//  INTERFACE.println();
}
//S88 *S88::fisrtS88=NULL;

